@@ -69,13 +69,17 @@ class IChannel:
     worker_name: str
     consume_pipe: IsolatePipes
     produce_pipe: IsolatePipes
+
     _listeners: Dict[str, List[EventListener]]
     _once_listeners: Dict[str, EventListener]
     _response_waiters: Dict[int, IMessageWaiter]
-    _inner_index_count: int
+
+    _before_event_middlewares: Dict[str, EventListener]
+    _after_event_middlewares: Dict[str, EventListener]
+    _inner_index: int
 
     def __init__(self, isolate_name: str):
-        self._inner_index_count = 0
+        self._inner_index = 0
         self.worker_name = isolate_name
         # self.debug = debug
         self._listeners = {}
@@ -88,9 +92,12 @@ class IChannel:
         return f"IChannel(name={self.worker_name})"
 
     async def produce(self, msg: IMessage):
-        self._inner_index_count += 1
-        msg.inner_index = self._inner_index_count
+        self._inner_index += 1
+        msg.inner_index = self._inner_index
         msg.sender = self.worker_name
+        before_middleware = self._before_event_middlewares.get(msg.message_type)
+        if before_middleware:
+            asyncio.create_task(before_middleware(msg))
         await self.produce_pipe.write(msg)
 
     async def sent_to_consume(self, msg: IMessage):
@@ -113,6 +120,9 @@ class IChannel:
             once_callback = self._once_listeners.get(r.message_type)
             if once_callback:
                 asyncio.create_task(self._exec_once(r, once_callback))
+            after_middleware = self._after_event_middlewares.get(r.message_type)
+            if after_middleware:
+                asyncio.create_task(after_middleware(r))
             response_waiter = self._response_waiters.get(r.inner_index)
             if response_waiter:
                 response_waiter.result = r
@@ -126,6 +136,19 @@ class IChannel:
         await callback(msg)
         self._once_listeners.pop(msg.message_type)
 
+    async def add_event_middleware(self, event_type: str, callback: EventListener, assign_to: str = "after"):
+        """
+        :param callback: callback to call
+        :param event_type: "event_message_type"
+        :param assign_to: "before, after"
+        :return: None
+        """
+        match assign_to:
+            case 'before':
+                self._before_event_middlewares.update({event_type: callback})
+            case "after":
+                self._after_event_middlewares.update({event_type: callback})
+
     async def produce_for_response(self, msg: GenericIMessage) -> Any:
         task = IMessageWaiter()
         await self.produce(msg)
